#include <assert.h>
#include <stdio.h>
#include <windows.h>
#include "common.h"
#include "options.h"

/**
 *  options.c:
 *      Loads and saves free memory indicator options. 
 */

static void remove_eol(char *buffer);

void init_option_pack(struct option_pack *option)
{
    option->fontname = NULL;
    option->fontsize = 0;
    option->windowpos.x = 0;
    option->windowpos.y = 0;
    option->windowsize.cx = 0;
    option->windowsize.cy = 0;
    option->opacity = 1.00;
    option->poll_interval = 0;
    option->rate_hist = 0;
    option->destroyed = false;
}

bool load_options(struct option_pack *option, FILE *fp)
{
    enum { LINEBUF_SIZE = 1024 };
    const bool False = false;
    bool retres = true;
    bool repstate; /* repeate state */
    char *line = NULL;
    enum { OPT_BEGIN, OPT_FONT, OPT_WNDPOS, OPT_WNDSZ, OPT_OPACITY,
        OPT_INTERVAL, OPT_HIST, OPT_END } state = OPT_BEGIN;
    char *p;
    char *newbuf = NULL;

    line = (char *)malloc(sizeof(char) * LINEBUF_SIZE);
    RAISEERR(line != NULL, "Failed to malloc in load_options.\n");
    while (fgets(line, LINEBUF_SIZE, fp) != NULL) {
        remove_eol(line);
        do {
            repstate = false;
            switch (state) {
            case OPT_BEGIN:
                state = OPT_FONT;
                repstate = true;
                break;
            case OPT_FONT:
                p = strchr(line, ',');
                if (p == NULL) {
                    goto lbl_format_error;
                }
                *p = '\0';
                newbuf = (char *)malloc(sizeof(char) *
                    get_min_power_of_two_ge(p - line + 1));
                RAISEERR(newbuf != NULL, "Failed to malloc in load_options.\n");
                strcpy(newbuf, line);
                p++;
                if (option->fontname != NULL) {
                    free(option->fontname);
                }
                option->fontname = newbuf;
                newbuf = NULL;
                if (sscanf(p, "%d", &option->fontsize) < 1) {
                    goto lbl_format_error;
                }
                state = OPT_WNDPOS;
                break;
            case OPT_WNDPOS:
                if (sscanf(line, "%ld,%ld", &option->windowpos.x,
                    &option->windowpos.y) < 2)
                {
                    goto lbl_format_error;
                }
                state = OPT_WNDSZ;
                break;
            case OPT_WNDSZ:
                if (sscanf(line, "%ld,%ld", &option->windowsize.cx,
                    &option->windowsize.cy) < 2)
                {
                    goto lbl_format_error;
                }
                state = OPT_OPACITY;
                break;
            case OPT_OPACITY:
                if (sscanf(line, "%lf", &option->opacity) < 1) {
                    goto lbl_format_error;
                }
                state = OPT_INTERVAL;
                break;
            case OPT_INTERVAL:
                if (sscanf(line, "%ld", &option->poll_interval) < 1) {
                    goto lbl_format_error;
                }
                state = OPT_HIST;
                break;
            case OPT_HIST:
                if (sscanf(line, "%d", &option->rate_hist) < 1) {
                    goto lbl_format_error;
                }
                state = OPT_END;
                break;
            case OPT_END:
                goto lbl_format_error;
            }
        } while (repstate);
    }
    if (state != OPT_END) {
        goto lbl_format_error;
    }
    goto lbl_after_loading;
lbl_format_error:
    RAISEERR(False, "Error option format in load_options.");
lbl_after_loading:
    ;
lbl_finally:
    if (newbuf != NULL) {
        free(newbuf);
    }
    if (line != NULL) {
        free(line);
    }
    return retres;
}

bool save_options(struct option_pack *option, FILE *fp)
{
    int retres = true;

    retres = retres && (fprintf(fp, "%s, %d\n", option->fontname,
        option->fontsize) != EOF);
    retres = retres && (fprintf(fp, "%ld, %ld\n", option->windowpos.x,
        option->windowpos.y) != EOF);
    retres = retres && (fprintf(fp, "%ld, %ld\n", option->windowsize.cx,
        option->windowsize.cy) != EOF);
    retres = retres && (fprintf(fp, "%.2lf\n", option->opacity) != EOF);
    retres = retres && (fprintf(fp, "%ld\n", option->poll_interval) != EOF);
    retres = retres && (fprintf(fp, "%d\n", option->rate_hist) != EOF);
    RAISEERR(retres, "Failed to write to file in save_options.\n");
lbl_finally:
    return (bool)retres;
}

void destroy_option_pack(struct option_pack *option)
{
    if (option == NULL) {
        return;
    }
    if (option->destroyed) {
        return;
    }
    if (option->fontname != NULL) {
        free(option->fontname);
    }
    option->destroyed = true;
}

static void remove_eol(char *buffer)
{
    char *p;

    for (p = buffer; *p != '\0'; p++) {
        if (*p == '\n') {
            break;
        }
    }
    if (*p != '\0') {
        *p = '\0';
    }
}  

